const fs = require('fs-extra')
const path = require('path')
const ora = require('ora')
const kebabCase = require('lodash.kebabcase')
const { print, run } = require('@toptal/davinci-cli-shared')

const skeletonLibMainPath = require.resolve('@toptal/davinci-skeleton')

const NODE_MODULES = /skeleton\/node_modules/g

const checkIfAppExists = (dest, appName) => {
  const appPath = path.join(dest, appName)

  if (fs.existsSync(appPath)) {
    print.red('Application with name', appName, 'already exists!')
    return true
  }

  return false
}

const copyApplicationSkeleton = async (dest, appName) => {
  const appDir = path.join(skeletonLibMainPath, '../..')
  const destDir = path.join(dest, appName)

  const spinner = ora('Copying skeleton...').start()

  const ignoreSkeletonFiles = src => !src.match(NODE_MODULES)

  try {
    await fs.copy(appDir, destDir, { filter: ignoreSkeletonFiles })
    spinner.succeed('Skeleton copied!')
  } catch (err) {
    spinner.fail()
    throw err
  }
}

const modifyPackageJson = async (dest, appName) => {
  const packageJsonFile = path.join(dest, appName, './package.json')

  const spinner = ora('Modifiying package.json...').start()

  try {
    const configData = fs.readFileSync(packageJsonFile)
    const config = JSON.parse(configData)

    config.name = `@toptal/${appName}`

    const modifiedConfigData = JSON.stringify(config, null, 2).concat('\n')

    fs.writeFileSync(packageJsonFile, modifiedConfigData)

    spinner.succeed('Package.json modified!')
  } catch (err) {
    spinner.fail()
    throw err
  }
}

const installDependencies = async appName => {
  const spinner = ora('Installing dependencies... ').start()

  try {
    await run(['yarn', '--cwd', appName])
    spinner.succeed('Dependencies installed!')
  } catch (err) {
    spinner.fail()
    throw new Error(err)
  }
}

const createCommand = async appName => {
  print.green('Creating', appName, '...')

  const currentRunningDir = process.cwd()

  try {
    if (checkIfAppExists(currentRunningDir, appName)) {
      return
    }

    // copy skeleton to destination directory
    await copyApplicationSkeleton(currentRunningDir, appName)

    // change package.json
    await modifyPackageJson(currentRunningDir, appName)

    // run yarn
    await installDependencies(appName)

    print.green('Application created.')
  } catch (err) {
    print.red(err)
  }
}

const newCommandCreator = {
  command: 'new <appName>',
  description: 'Create new application',
  action: ({ appName }) => {
    const formattedAppName = kebabCase(appName)

    createCommand(formattedAppName)
  }
}

module.exports = newCommandCreator
